_PROVIDERS = dict()
_CALLED = dict()

def get_calls(obj):
    if obj not in _CALLED:
        _CALLED[obj] = list()
    return _CALLED[obj]
    
def was_called(obj, method):
    called = get_calls(obj)
    if method in called:
        return True
    else:
        return False

def set_called(obj, method):
    called = get_calls(obj)
    called.append(method)

def get_providers(module):
    if module not in _PROVIDERS:
        _PROVIDERS[module] = dict()
    return _PROVIDERS[module]

def get_provider_for(module, attribute):
    providers = get_providers(module)
    if attribute in providers:
        return providers[attribute]
    return None

def set_provider(module, method, attribute):
    providers = get_providers(module)
    providers[attribute] = method

def requires(attributes):
    if not isinstance(attributes, (list, tuple)):
        attributes = [attributes]

    def wrapped_requiring(method):
        if method.__module__ == 'lint.depends' and method.__name__ == 'with_provides':
            raise Exception("@required must come after @provides")
            
        def with_required(self):
            for a in attributes:
                provider = get_provider_for(method.__module__, a)
                if provider is None:
                    raise Exception("No provider found for %s in module: %s" % (a, method.__module__))
                if not was_called(self, provider) or not hasattr(self, a):
                    set_called(self, provider)
                    provider(self)
            return method(self)
        with_required.__module__ = method.__module__
        with_required.__name__ = method.__name__
        return with_required
    return wrapped_requiring
    
def provides(attributes):
    if not isinstance(attributes, (list, tuple)):
        attributes = [attributes]
    
    def wrapped_provider(method):
        for a in attributes:
            set_provider(method.__module__, method, a)
        def with_provides(self):
            set_called(self, method)
            return method(self)
        with_provides.__module__ = method.__module__
        with_provides.__name__ = method.__name__
        return with_provides
    return wrapped_provider
